import os 
import json 
import openpyxl
import csv
from unittest import TestCase


from crawlfish.crawler.save import ListDataSaver
from crawlfish import CSVSaveOption , ExcelSaveOption , JSONSaveOption,JSONFileSaveOption

test_scraped_data = [
			["NAME" , "AGE" , "EMAIL"],
			["John" , "44" , "john@gmail.com"],
			["Mao" , "66" , "maozedong@gmail.com"],
			["Heather" , "77" , "heather@gmail.com"],
			["Mark" , "34" , "mark@gmail.com"],

		]
test_scraped_data_dict = {
		"John":{"AGE":"44" , "EMAIL" : 'john@gmail.com'},
		"Mao":{"AGE":"66" , "EMAIL" : 'maozedong@gmail.com'},
		"Heather":{"AGE":"77" , "EMAIL" : 'heather@gmail.com'},
		"Mark":{"AGE":"34" , "EMAIL" : 'mark@gmail.com'},

}
test_scraped_data_json = test_scraped_data_dict
data_saver = ListDataSaver()

class TestListDataSaver(TestCase):

	def test_saving_to_csv_file(self):
		test_file = "tests/test_data/test_scraped_data.csv"
		if os.path.isfile(test_file):
			os.remove(test_file)
		data_saver.to_csv_file(test_scraped_data , test_file)
		self.assertTrue(os.path.isfile(test_file))
		os.remove(test_file)
		# Test overwrite
			
		data_to_append = [["NAME" , "AGE" , "EMAIL"], ["Heisenberg" , "67" , "albrqq@gmail.com"],]
		expected_append_data = test_scraped_data + data_to_append[1:]
		data_saver.to_csv_file(test_scraped_data, test_file)
		data_saver.to_csv_file(data_to_append , test_file, overwrite = False)
		with open(test_file , 'r') as f:
			reader = csv.reader(f)
			rows = [row for row in reader ]
			self.assertTrue(rows == expected_append_data)


	def test_saving_to_spreadsheet_file(self):
		test_file = "tests/test_data/test_scraped_data.xlsx"
		if os.path.isfile(test_file):
			os.remove(test_file)
		data_saver.to_spreadsheet_file(test_scraped_data , test_file , sheet_name = "TestSheet1" , overwrite = True)
		self.assertTrue(os.path.isfile(test_file))
		workbook  = openpyxl.load_workbook(test_file)
		sheet = workbook["TestSheet1"]
		sheet_values = [ [ i for i in row] for row in sheet.values  ]
		self.assertTrue(sheet_values == test_scraped_data)
		os.remove(test_file)
		# write and then append data to first sheet and test data , test programatically and check manually 
		data_saver.to_spreadsheet_file(test_scraped_data , test_file , sheet_name = "TestSheet1" , overwrite = True)
		data_saver.to_spreadsheet_file(test_scraped_data , test_file , overwrite = False , sheet_name = "TestSheet1")
		workbook  = openpyxl.load_workbook(test_file)
		sheet = workbook["TestSheet1"]
		written_data = [ [ item for item in row ] for row in sheet.values ]# remove the tuples from sheet.values
		test_overwite_data = test_scraped_data + test_scraped_data[1:]
		self.assertTrue(written_data == test_overwite_data)
		# write to new sheet and test ovwrite ,  test programatically and check manually 
		data_saver.to_spreadsheet_file(test_scraped_data , test_file , sheet_name = "TestSheet2" )
		data_saver.to_spreadsheet_file(test_scraped_data[:-1] , test_file , sheet_name = "TestSheet2" )
		workbook = openpyxl.load_workbook(test_file)
		sheet = workbook["TestSheet2"]
		sheet_values = [ [ i for i in row] for row in sheet.values  ]
		self.assertTrue(sheet_values == test_scraped_data[:-1])
	
	def test_saving_to_dict(self):
		result = data_saver.to_dict(test_scraped_data , key_index = 0)
		self.assertTrue(result == test_scraped_data_dict)

	def test_saving_to_json(self):
		result = data_saver.to_json(test_scraped_data, key_index = 0 )
		self.assertTrue( result == test_scraped_data_json )

	def test_saving_to_json_file(self):
		test_file = "tests/test_data/data_json.json"
		if os.path.isfile(test_file):
			os.remove(test_file)
		result = data_saver.to_json_file(test_scraped_data, file = test_file )
		with open(test_file , "r" ) as f:
			data = json.load(f)
			self.assertTrue(data== test_scraped_data_json)
		self.assertTrue( result == test_scraped_data_json )
		self.assertTrue(os.path.isfile(test_file))
		# test overwite 
		os.remove(test_file)
		json_data_to_append =  [
			["NAME" , "AGE" , "EMAIL"],
			["Heisenberg" , '67' , "albrqq@gmail.com"],]
		expected_append_data = test_scraped_data_dict | {
			 'Heisenberg': {'AGE': '67', 'EMAIL': 'albrqq@gmail.com'}}
		expected_append_data_json = json.dumps(expected_append_data)
		result = data_saver.to_json_file(test_scraped_data, file = test_file )
		append_result =  data_saver.to_json_file(json_data_to_append, file = test_file , overwrite = False )
		with open(test_file , "r" ) as f:
			data = json.load(f)
			self.assertTrue(expected_append_data_json == json.dumps(data) )
		os.remove(test_file)


	def test_save_by_option(self):



		# csv 
		test_file = "tests/test_data/data.csv"
		save_option  = CSVSaveOption(test_file)
		result = data_saver.save_by_option(test_scraped_data , save_option)
		self.assertTrue(os.path.isfile(test_file))
		self.assertTrue(result == test_scraped_data)
		with open(test_file , "r") as f:
			reader = csv.reader(f)
			rows = [row for row in reader]
			self.assertTrue(rows == test_scraped_data)
		os.remove(test_file)

		# excel
		test_file = "tests/test_data/excel.xlsx"
		save_option = ExcelSaveOption(test_file , "testsheet" )
		result = data_saver.save_by_option(test_scraped_data , save_option)
		self.assertTrue(result == test_scraped_data)
		workbook  = openpyxl.load_workbook(test_file)
		sheet = workbook["testsheet"]
		sheet_values = [ [ i for i in row] for row in sheet.values  ]
		self.assertTrue(sheet_values == test_scraped_data)
		os.remove(test_file)

		# json 
		save_option = JSONSaveOption(  )
		result = data_saver.save_by_option(test_scraped_data , save_option)
		self.assertTrue(test_scraped_data_dict == result)

		# json file
		test_file = "tests/test_data/json.json"
		save_option = JSONFileSaveOption(test_file  )
		result = data_saver.save_by_option(test_scraped_data , save_option)
		self.assertTrue(test_scraped_data_dict == result)
		with open(test_file , "r" ) as f:
			data = json.load(f)
			self.assertTrue(data == test_scraped_data_json)
		os.remove(test_file)






from crawlfish.crawler.get_base_url import get_base_url

from unittest import TestCase 


class Test_explore_website(TestCase):

	def test_metadata(self):
		test_link = "https://stackoverflow.com/questions/"
		url = get_base_url(test_link)
		print("url " , url)
		self.assertTrue(url == 'https://stackoverflow.com' )




from unittest import TestCase 

from crawlfish.crawler import ensure_html_url







class Test_gensure_is_html_url(TestCase):

	def test_func(self):
		test_url = "https://site.com/a/b/c/d.png?r=45,dfjd=er"
		result =  ensure_html_url(test_url)
		self.assertFalse(result)




from unittest import TestCase

from crawlfish.html.get_first_text_value import get_first_text_value

from .test_soups import elements_to_find_soup



class Test_get_first_text_value(TestCase):

	def test_finding_correct_values(self):

		soup = elements_to_find_soup
		# simple scenario
		value = get_first_text_value(soup , "a" )
		expected_value = "nolinkhere"
		self.assertTrue(expected_value == value)
		




from unittest import TestCase

from crawlfish.crawler.extract_html_meta import extract_html_meta

from .test_soups import test_extract_meta_soup


class TestExtractHtmlMEtafunc(TestCase):
	def test_func(self):
		test_soup = test_extract_meta_soup
		data = extract_html_meta(test_soup)
		print("Extracted test meta " , data)
		self.assertTrue('viewport' in data )
		self.assertTrue(data['viewport'] == '200x200')



import os
import csv
from unittest import TestCase 

from crawlfish.crawlfish_utils import check_file
from crawlfish.crawler.crawler import UrlsCrawler 
from crawlfish import CSVSaveOption


 

TEST_WEBPAGE_DATA_DICT = {
	"https://company_website1.com":["company_website1" , "CEO1"],
	"https://company_website2.com":["company_website2" , "CEO2"],
	"https://company_website3.com":["company_website3" , "CEO3"],
	"https://company_website4.com":["company_website4" , "CEO4"],

} 
TEST_URLS = list(TEST_WEBPAGE_DATA_DICT.keys())

TEST_WEBPAGE_DATA_LIST = list(TEST_WEBPAGE_DATA_DICT.values())
WEBPAGE_DATA_HEADERS = ["COMPANY_NAME" , "CEO_NAME"]

test_csv_output_file ="tests/test_data/test_scraped_data.csv" 
test_csv_save_option = CSVSaveOption(test_csv_output_file)
def test_scrape_url(*args , **kwargs):
	# print("Scraping link " )
	# print(args  , kwargs)
	return TEST_WEBPAGE_DATA_DICT[args[0]]


class Test_UrlsCrawler(TestCase):

	def test_simple_Scraping_Scenario(self):
		crawler = UrlsCrawler( )
		if os.path.isfile(test_csv_output_file):
			os.remove(test_csv_output_file)
		crawler.save_option = test_csv_save_option
		data = crawler.scrape_urls(
			TEST_URLS , test_scrape_url , WEBPAGE_DATA_HEADERS , 
			    ) 
		self.assertTrue(data == TEST_WEBPAGE_DATA_LIST)
		# CHECK IF DATA IS WRITTEN OK 
		with open(crawler.save_option.file, "r") as f:
			reader = csv.reader(f)
			rows = [[i for i in row] for row in reader]
			self.assertTrue(rows[1:] == TEST_WEBPAGE_DATA_LIST )






import os 


from unittest import TestCase

from crawlfish.crawler import download_video , download_videos

test_video_link_1 = "https://www.sample-videos.com/video321/mp4/720/big_buck_bunny_720p_2mb.mp4"
test_video_link_2 = "https://www.sample-videos.com/video321/mp4/720/big_buck_bunny_720p_5mb.mp4"

test_output_folder = "tests/test_data/test_video_download"

class TestDownloadVideo(TestCase):

	def test_download_video_function(self):
		# check if the file to be downloadd is found 
		expected_size = 2107842
		expected_path = os.path.join(test_output_folder , "big_buck_bunny_720p_2mb.mp4")
		if os.path.isfile(expected_path):
			os.remove(expected_path)
		file_name , data = download_video(test_video_link_1 , output_folder = test_output_folder)	
		print("FILE SIZE " , os.path.getsize(expected_path))
		self.assertTrue(os.path.isfile(expected_path))
		self.assertTrue(len(data) == expected_size)
		self.assertTrue(os.path.getsize(expected_path) == expected_size)


		# test providing custom file name 
		expected_path = os.path.join(test_output_folder , "myvideo.mp4")
		if os.path.isfile(expected_path):
			os.remove(expected_path)
		file_name , data = download_video(test_video_link_1 , output_folder = test_output_folder ,file_name = "myvideo")	
		self.assertTrue(os.path.isfile(expected_path))
		self.assertTrue(len(data) == expected_size)
		self.assertTrue(os.path.getsize(expected_path) == expected_size)




from unittest import TestCase

from crawlfish import ElementFinder 

from .test_soups import elements_to_find_soup


ef = ElementFinder(elements_to_find_soup)

class Test_FindingElements(TestCase):

	def test_finding_with_text_match(self):
		# We should get 3 elements that have the text 'randomperson' in them 
		elements = ef.find_elements(tag_name = "p" , text_match = "randomperson")
		self.assertTrue(len(elements) == 3 )
		print(elements)
		elements = ef.find_elements(text_match = "randomperson")
		self.assertTrue(len(elements) == 5 ) # expeting 5 elements 


	def test_finding_by_attribute_value_only(self):
		# we expect 5 elements with an attribute value of 'ocean'
		elements  =ef.find_elements(attribute_value = "ocean")
		self.assertTrue(len(elements) == 6)

	def test_finding_by_tag_name_only(self):
		# expecting 3 <p> elements 
		elements = ef.find_elements(tag_name = "p")
		self.assertTrue(len(elements) == 3 )

	def test_finding_by_attribute_only(self):
		# we expect 6  elements with the 'class' attribute present  
		elements  =ef.find_elements(attribute = "class")
		self.assertTrue(len(elements) == 6)

	def test_finding_combos(self):
		elements = ef.find_elements(tag_name="h3"  ,  attribute_value = "heading")
		# expecting 2 <h3> elements 
		self.assertTrue(len(elements) == 2)
		# expecting 2 elements
		elements = ef.find_elements(text_match = "orbit" , attribute_value="planet")
		self.assertTrue(len(elements) == 2)

	def test_finding_with_num_kwarg(self):
		elements = ef.find_elements(tag_name="a" , num =1 )
		# expecting 1 <a> elements 
		self.assertTrue(len(elements) == 1)

	def test_finding_with_attr_value_attr_tag_name(self):
		elements = ef.find_elements(tag_name="div" ,attribute = "id" , attribute_value = "name")
		# expecting   2 elements 
		self.assertTrue(len(elements) == 2)

	def test_complex_combo(self):
		html = """
		<!DOCTYPE html>
		<html>
		<head>
			<title> Let's find elements </title>
		</head>
		<body>

		<div id="person" class="person-container" > 
			 Someone had an email and it was someemail@gmail.com , yeah , his name was Ail Someem
		</div>

		<div id="" class="" ></div>
		<div id="" class="" ></div>
		<div id="" class="" ></div>

		<div id="" class="email-div" ></div>

		<p id="email-p" >validemail@gmail.com</p>
		<p id="email-p" ></p>
		<p id="" class="email-p"></p>
		<p id="" class="email-p"></p>
		<div id="" class="email-div" >Anothervalidemail@gmail.com</div>
		<div id="" class="email-div" ></div>

		<p id="" class="email-p"></p>

		<a id="" class="" ></a>
		<a id="" class="" ></a>
		<a id="" class="" ></a>
		<a id="" class="" ></a>
		<a id="" class="" ></a>



		</body>
	</html>

		"""
		email_regex = r"[\w\.-]+@[\w\.-]+(\.[\w]+)+" 
		finder = ElementFinder(html)
		elements = finder.find_elements(  attributes = [ "id" , "class"] , attribute_values = ['email' , ] ,
			 tag_names = [ 'div' , 'p'] ,text_matches = [email_regex, ]    )
		expected_number_of_elements = 3
		self.assertTrue(len(elements) == expected_number_of_elements)


# TEST DATA FOR UUH, TESTING 

import os 
from bs4 import BeautifulSoup



base_dir = os.path.split(os.path.abspath(__file__))[0]

def read_file(file):
	data = ''
	file = os.path.join(base_dir , file)
	with open(file , 'r') as f:
		data = f.read()
	return data

make_soup = lambda html: BeautifulSoup(html , "html.parser")


test_extract_meta_soup  = make_soup( read_file("test_extract_meta_soup.html"))

grocery_list_soup = make_soup( read_file("grocery_list.html"))
elements_to_find_soup = make_soup( read_file("test_elements_to_find.html"))
test_internal_static_soup = make_soup(read_file("test_internal_static_code.html"))




from datetime import datetime

from crawlfish.crawler.explore_website import explore_website

from unittest import TestCase 

test_output_folder = "tests/test_data/test_explore_website_data"

test_link = 'https://www.countryle.com/'
'https://lakeviewequipment.com/'
'https://www.sample-videos.com'
"https://www.zetech.ac.ke/"

class Test_explore_website(TestCase):

	def test_func(self):
		wr = explore_website(test_link)
		wr.save(output_folder = test_output_folder)
		print("Here is the report")
		print(wr.page_reports.keys())

 